@@ -27,6 +27,10 @@
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
+#define _LARGEFILE_SOURCE 1
+
+#include "mpg321.h"
+
 #include <unistd.h>
 #include <sys/mman.h>
 #include <stdio.h>
@@ -37,8 +41,6 @@
 #include <unistd.h>
 #include <fcntl.h>
 
-#include "mpg321.h"
-
 unsigned long current_frame=0;
 
 enum mad_flow read_from_mmap(void *data, struct mad_stream *stream)
@@ -445,11 +447,20 @@ void scan(void const *ptr, ssize_t len, buffer *buf)
             break;
         }
             
+       /* In quiet mode, don't scan the whole file for length since
+           it takes so long on slow file systems like sshfs. cplay
+           calls mpg321 for playback in quiet mode. */
+        if ((options.opt & MPG321_QUIET_PLAY) && (buf->num_frames > 20))
+        {
+            break;
+        }
+
         mad_timer_add(&buf->duration, header.duration);
     }
 
     if (!is_vbr)
     {
+       if (header.bitrate!=0 && MAD_NSBSAMPLES(&header)!=0) {
         double time = (len * 8.0) / (header.bitrate); /* time in seconds */
         double timefrac = (double)time - ((long)(time));
         long nsamples = 32 * MAD_NSBSAMPLES(&header); /* samples per frame */
@@ -458,6 +469,7 @@ void scan(void const *ptr, ssize_t len, buffer *buf)
         buf->num_frames = (long) (time * header.samplerate / nsamples);
 
         mad_timer_set(&buf->duration, (long)time, (long)(timefrac*100), 100);
+       }
     }
         
     else if (has_xing)
